/// <reference types="node" />
import { SpawnMessage, GameSettings, BaseGameDataMessage, BaseRootMessage, ComponentSpawnData, AllGameSettings, PlayerJoinData } from "@skeldjs/protocol";
import { GameOverReason, GameState } from "@skeldjs/constant";
import { EventEmitter, ExtractEventTypes } from "@skeldjs/events";
import { AirshipStatus, AprilShipStatus, CustomNetworkTransform, GameData, GameDataEvents, MiraShipStatus, LobbyBehaviour, LobbyBehaviourEvents, MeetingHud, MeetingHudEvents, PolusShipStatus, PlayerControl, PlayerPhysics, SkeldShipStatus, ShipStatusEvents, VoteBanSystem, VoteBanSystemEvents, PlayerIDResolvable, InnerShipStatus } from "./objects";
import { Networkable, NetworkableEvents, NetworkableConstructor } from "./Networkable";
import { PlayerData, PlayerDataEvents } from "./PlayerData";
import { HostableOptions as HostableConfig } from "./misc/HostableOptions";
import { RoomEndGameIntentEvent, RoomFixedUpdateEvent, RoomGameEndEvent, RoomGameStartEvent, RoomSetPrivacyEvent } from "./events";
import { EndGameIntent } from "./endgame";
import { BaseRole } from "./roles";
export declare type RoomID = string | number;
export declare type PlayerDataResolvable = number | PlayerData | PlayerControl | PlayerPhysics | CustomNetworkTransform;
export declare type PrivacyType = "public" | "private";
export interface SpawnObject {
    type: number;
    ownerId: number;
    flags: number;
    components: Networkable<any, any>[];
}
export declare type AnyNetworkable<RoomType extends Hostable = Hostable> = AirshipStatus<RoomType> | AprilShipStatus<RoomType> | CustomNetworkTransform<RoomType> | GameData<RoomType> | MiraShipStatus<RoomType> | LobbyBehaviour<RoomType> | MeetingHud<RoomType> | PolusShipStatus<RoomType> | PlayerControl<RoomType> | PlayerPhysics<RoomType> | SkeldShipStatus<RoomType> | VoteBanSystem<RoomType>;
export declare type HostableEvents<RoomType extends Hostable = Hostable> = NetworkableEvents<RoomType> & PlayerDataEvents<RoomType> & GameDataEvents<RoomType> & LobbyBehaviourEvents<RoomType> & MeetingHudEvents<RoomType> & ShipStatusEvents<RoomType> & VoteBanSystemEvents<RoomType> & ExtractEventTypes<[
    RoomEndGameIntentEvent<RoomType>,
    RoomFixedUpdateEvent<RoomType>,
    RoomGameEndEvent<RoomType>,
    RoomGameStartEvent<RoomType>,
    RoomSetPrivacyEvent<RoomType>
]>;
export declare type GetHostableEvents<T extends Hostable<HostableEvents>> = T extends Hostable<infer X> ? X : never;
/**
 * Represents an object capable of hosting games.
 *
 * See {@link HostableEvents} for events to listen to.
 */
export declare class Hostable<T extends HostableEvents = any> extends EventEmitter<T> {
    config: HostableConfig;
    /**
     * Whether or not this room has been destroyed.
     */
    protected _destroyed: boolean;
    protected last_fixed_update: number;
    protected _interval?: NodeJS.Timeout;
    protected _incrNetId: number;
    /**
     * A list of all objects in the room.
     */
    objectList: Networkable[];
    /**
     * The players in the room.
     */
    players: Map<number, PlayerData<this>>;
    /**
     * The networkable components in the room.
     */
    netobjects: Map<number, Networkable<any, NetworkableEvents, this>>;
    /**
     * The current message stream to be sent to the server on fixed update.
     */
    messageStream: BaseGameDataMessage[];
    /**
     * The code of the room.
     */
    code: number;
    /**
     * The state that the game is currently in.
     */
    gameState: GameState;
    /**
     * The ID of the host of the room.
     */
    hostId: number;
    /**
     * The settings of the room.
     */
    settings: GameSettings;
    /**
     * The current start counter for the room.
     */
    counter: number;
    /**
     * The privacy state of the room.
     */
    privacy: PrivacyType;
    /**
     * An instance of the ship status in the room. Spawned when a game is started
     * and represents the current map.
     *
     * It is possible for more than one ship status to be spawned, however this
     * property is set and cleared for simplicity.
     *
     * See {@link Hostable.objectList} if you're looking to find all ship status
     * objects that have been spawned.
     */
    shipStatus: InnerShipStatus<this> | undefined;
    /**
     * An instance of the meeting hud in the room. Spawned when a meeting is
     * started.
     *
     * It is possible for more than one meeting hud to be spawned, however this
     * property is set and cleared for simplicity.
     *
     * See {@link Hostable.objectList} if you're looking to find all meeting hud
     * objects that have been spawned.
     */
    meetingHud: MeetingHud<this> | undefined;
    /**
     * An instance of the lobby behaviour in the room. Spawned when the room is
     * currently in the lobby waiting for a game to start.
     *
     * It is possible for more than one lobby behaviour to be spawned, however this
     * property is set and cleared for simplicity.
     *
     * See {@link Hostable.objectList} if you're looking to find all lobby behaviour
     * objects that have been spawned.
     */
    lobbyBehaviour: LobbyBehaviour<this> | undefined;
    /**
     * An instance of the game data in the room. Contains information about players
     * such as their name, colour, hat etc.
     *
     * It is possible for more than one game data to be spawned, however this
     * property is set and cleared for simplicity.
     *
     * See {@link Hostable.objectList} if you're looking to find all game data
     * objects that have been spawned.
     */
    gameData: GameData<this> | undefined;
    /**
     * An instance of the voting ban system in the room. Used as a utility object
     * for handling kick and ban votes.
     *
     * It is possible for more than one vote ban system to be spawned, however this
     * property is set and cleared for simplicity.
     *
     * See {@link Hostable.objectList} if you're looking to find all vote ban system
     * objects that have been spawned.
     */
    voteBanSystem: VoteBanSystem<this> | undefined;
    /**
     * A map of spawn type -> objects used in the protocol. Useful to register
     * custom INO (inner net objects) such as replicating behaviour from a client
     * mod, see {@link Hostable.registerPrefab}.
     */
    registeredPrefabs: Map<number, NetworkableConstructor<any>[]>;
    /**
     * All roles that can be assigned to players. See {@link Hostable.registerRole}.
     */
    registeredRoles: Map<number, typeof BaseRole>;
    /**
     * Every end game intent that should happen in a queue to be or not to be
     * canceled.
     *
     * The first item of each element being the name of the end game intent and
     * the second being metadata passed with it.
     *
     * See {@link RoomEndGameIntentEvent} to cancel an end game intent, or see
     * {@link Hostable.registerEndGameIntent} to register your own.
     *
     * See {@link RoomGameEndEvent} to listen for a game actually ending.
     */
    endGameIntents: EndGameIntent<any>[];
    constructor(config?: HostableConfig);
    destroy(): void;
    getNextNetId(): number;
    /**
     * The current client in the room.
     */
    get myPlayer(): PlayerData<this> | undefined;
    /**
     * The host of the room.
     */
    get host(): PlayerData<this> | undefined;
    get destroyed(): boolean;
    /**
     * Whether or not the current client is the host of the room.
     */
    get hostIsMe(): boolean;
    broadcast(gamedata: BaseGameDataMessage[], payloads?: BaseRootMessage[], include?: (PlayerData | number)[], exclude?: (PlayerData | number)[], reliable?: boolean): Promise<void>;
    FixedUpdate(): Promise<void>;
    /**
     * Resolve a player by some identifier.
     * @param player The identifier to resolve to a player.
     * @returns The resolved player.
     * @example
     *```typescript
     * // Resolve a player by their client id.
     * const player = room.resolvePlayer(11013);
     * ```
     */
    resolvePlayer(player: PlayerDataResolvable): PlayerData<this> | undefined;
    /**
     * Resolve a player ID by some identifier.
     * @param player The identifier to resolve to a player ID.
     * @returns The resolved player ID.
     */
    resolvePlayerId(player: PlayerIDResolvable): number | undefined;
    /**
     * Resolve a client id by some identifier.
     * @param player The identifier to resolve to a client ID.
     * @returns The resolved client ID.
     */
    resolvePlayerClientID(player: PlayerDataResolvable): number | undefined;
    /**
     * Set the code of the room.
     * @example
     *```typescript
     * room.setCode("ABCDEF");
     * ```
     */
    setCode(code: RoomID): void;
    protected _setPrivacy(privacy: PrivacyType): void;
    /**
     * Change the the privacy of the room.
     * @param tag The tag to change.
     * @param value The new value of the tag.
     * @example
     *```typescript
     * room.setAlterGameTag(AlterGameTag.ChangePrivacy, 1); // 0 for private, 1 for public.
     * ```
     */
    setPrivacy(privacy: PrivacyType): Promise<void>;
    /**
     * Change the settings of the room. If the host, it will broadcast these changes.
     * @param settings The settings to set to (Can be partial).
     * @example
     *```typescript
     * room.syncSettings({
     *   crewmateVision: 0.5,
     *   votingTime: 120
     * });
     * ```
     */
    setSettings(settings: Partial<AllGameSettings>): void;
    spawnNecessaryObjects(): void;
    /**
     * Set the host of the room. If the current client is the host, it will conduct required host changes.
     * e.g. Spawning objects if they are not already spawned.
     * @param host The new host of the room.
     */
    setHost(host: PlayerDataResolvable): Promise<void>;
    /**
     * Handle when a client joins the game.
     * @param clientId The ID of the client that joined the game.
     */
    handleJoin(joinInfo: PlayerJoinData): Promise<PlayerData<this>>;
    checkPlayersRemaining(): void;
    /**
     * Handle when a client leaves the game.
     * @param resolvable The client that left the game.
     */
    handleLeave(resolvable: PlayerDataResolvable): Promise<PlayerData<this> | null>;
    /**
     * Handle when the game is started.
     */
    handleStart(): Promise<void>;
    protected _handleEnd(reason: GameOverReason): Promise<void>;
    /**
     * Handle when the game is ended.
     * @param reason The reason for why the game ended.
     */
    handleEnd(reason: GameOverReason): Promise<void>;
    endGame(reason: GameOverReason): Promise<void>;
    /**
     * Handle a client readying up.
     * @param player The client that readied.
     */
    handleReady(player: PlayerDataResolvable): Promise<void>;
    /**
     * Spawn a component (Not broadcasted to all clients, see {@link Hostable.spawnPrefabOfType}).
     * @param component The component being spawned.
     * @example
     * ```typescript
     * const meetinghud = new MeetingHud(
     *   this,
     *   this.getNextNetId(),
     *   ownerId,
     *   {
     *     dirtyBit: 0,
     *     states: new Map(),
     *   }
     * );
     *
     * this.spawnComponent(meetinghud);
     * ```
     */
    spawnComponent(component: Networkable<any, NetworkableEvents, this>): void;
    protected _despawnComponent(component: Networkable<any>): void;
    /**
     * Despawn a component.
     * @param component The component being despawned.
     * @example
     *```typescript
     * room.despawnComponent(room.meetinghud);
     * ```
     */
    despawnComponent(component: Networkable<any, NetworkableEvents, this>): void;
    /**
     * Get an available player ID.
     * @returns The player ID that was found.
     * @example
     *```typescript
     * // Get an available player ID and add it to the gamedata.
     * const playerId = room.getAvailablePlayerID();
     * room.gamedata.add(playerId);
     * ```
     */
    getAvailablePlayerID(): number;
    getExistingObjectSpawn(): SpawnMessage[];
    protected spawnPrefab(spawnType: number, spawnPrefab: NetworkableConstructor<any>[], ownerId: number | PlayerData | undefined, flags?: number, componentData?: (any | ComponentSpawnData)[], doBroadcast?: boolean, doAwake?: boolean): Networkable<any, any, this> | undefined;
    /**
     * Spawn a prefab of an object.
     * @param spawnType The type of object to spawn.
     * @param owner The owner or ID of the owner of the object to spawn.
     * @returns The object that was spawned.
     * @example
     *```typescript
     * room.spawnPrefab(SpawnType.Player, client.myPlayer);
     * ```
     */
    spawnPrefabOfType(spawnType: number, ownerId: number | PlayerData | undefined, flags?: number, componentData?: (any | ComponentSpawnData)[], doBroadcast?: boolean, doAwake?: boolean): Networkable<any, any, this> | undefined;
    /**
     * Create a fake player in the room that doesn't need a client to be connected
     * to own it.
     *
     * To dispose of the player, use `player.despawn()`.
     * @param isNew Whether or not the player should be seen jumping off of the seat in the lobby.
     * @returns The fake player created.
     */
    createFakePlayer(isNew?: boolean): PlayerData<this>;
    /**
     * Register a custom INO spawn object by its spawn type. Can also override
     * built-in objects.
     * @param spawnType The spawn type of the component as an integer.
     * @param components The components in the object. The first component should
     * be the main object which will inherit the rest of the components.
     */
    registerPrefab(spawnType: number, components: NetworkableConstructor<Networkable>[]): void;
    /**
     * Register a role that can be assigned to a player.
     * @param roleCtr The role to register to the room.
     */
    registerRole(roleCtr: typeof BaseRole): void;
    /**
     * Get a player by their player ID.
     * @param playerId The player ID of the player.
     * @returns The player that was found, or null if they do not exist.
     * @example
     * ```typescript
     * const player = room.getPlayerByPlayerId(1);
     * ```
     */
    getPlayerByPlayerId(playerId: number): PlayerData<this> | undefined;
    getPlayerControlByPlayerId(playerId: number): PlayerControl<this> | undefined;
    /**
     * Get a player by one of their components' netIds.
     * @param netId The net ID of the component of the player to search.
     * @returns The player that was found, or null if they do not exist.
     * @example
     * ```typescript
     * const player = room.getPlayerByNetId(34);
     * ```
     */
    getPlayerByNetId(netId: number): PlayerData<this> | undefined;
    findPlayersWithName(displayName: string): PlayerData<this>[];
    /**
     * Register an intent to end the game.
     * @param endGameIntent The intention to end the game.
     */
    registerEndGameIntent(endGameIntent: EndGameIntent<any>): void;
    murderIsValid(murderer: PlayerData, victim: PlayerData): boolean;
    /**
     * How often a FixedUpdate should be called.
     */
    static FixedUpdateInterval: number;
}
//# sourceMappingURL=Hostable.d.ts.map