"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlayerControl = void 0;
const util_1 = require("@skeldjs/util");
const protocol_1 = require("@skeldjs/protocol");
const constant_1 = require("@skeldjs/constant");
const events_1 = require("../events");
const Networkable_1 = require("../Networkable");
const LobbyBehaviour_1 = require("./LobbyBehaviour");
const component_1 = require("./component");
const systems_1 = require("../systems");
const endgame_1 = require("../endgame");
const roles_1 = require("../roles");
/**
 * Represents a player object for interacting with the game and other players.
 *
 * See {@link PlayerControlEvents} for events to listen to.
 */
class PlayerControl extends Networkable_1.Networkable {
    constructor(room, spawnType, netId, ownerid, flags, data) {
        var _a;
        super(room, spawnType, netId, ownerid, flags, data);
        this.lastStartCounter = 0;
        (_a = this.isNew) !== null && _a !== void 0 ? _a : (this.isNew = true);
        this.playerId || (this.playerId = 0);
        this.player = this.owner;
        this.protectedByGuardian = false;
        this._protectedByGuardianTime = 0;
    }
    Awake() {
        this.playerId || (this.playerId = this.room.getAvailablePlayerID());
        if (this.room.gameData) {
            this.room.gameData.add(this.playerId);
        }
        if (this.isNew) {
            if (this.room.lobbyBehaviour) {
                const spawnPosition = LobbyBehaviour_1.LobbyBehaviour.spawnPositions[this.playerId % LobbyBehaviour_1.LobbyBehaviour.spawnPositions.length];
                const offsetted = spawnPosition
                    .add(spawnPosition.negate().normalize().div(4));
                this.getComponent(component_1.CustomNetworkTransform).snapTo(offsetted, false);
            }
            else if (this.room.shipStatus) {
                const spawnPosition = this.room.shipStatus.getSpawnPosition(this.playerId, true);
                this.getComponent(component_1.CustomNetworkTransform).snapTo(spawnPosition, false);
            }
        }
    }
    getComponent(component) {
        if (this.spawnType === constant_1.SpawnType.Player) {
            if (component === PlayerControl) {
                return this.components[0];
            }
            if (component === component_1.PlayerPhysics) {
                return this.components[1];
            }
            if (component === component_1.CustomNetworkTransform) {
                return this.components[2];
            }
        }
        return undefined;
    }
    Deserialize(reader, spawn = false) {
        if (spawn) {
            this.isNew = reader.bool();
        }
        this.playerId = reader.uint8();
    }
    Serialize(writer, spawn = false) {
        if (spawn) {
            writer.bool(this.isNew);
            this.isNew = false;
        }
        writer.uint8(this.playerId);
        return true;
    }
    FixedUpdate(delta) {
        if (this.protectedByGuardian) {
            this._protectedByGuardianTime -= delta;
            if (this._protectedByGuardianTime <= 0) {
                this._protectedByGuardianTime = 0;
                this.removeProtection(true);
            }
        }
    }
    async HandleRpc(rpc) {
        switch (rpc.messageTag) {
            case constant_1.RpcMessageTag.CompleteTask:
                await this._handleCompleteTask(rpc);
                break;
            case constant_1.RpcMessageTag.SyncSettings:
                await this._handleSyncSettings(rpc);
                break;
            case constant_1.RpcMessageTag.CheckName:
                if (this.room.hostIsMe) {
                    await this._handleCheckName(rpc);
                }
                break;
            case constant_1.RpcMessageTag.CheckColor:
                if (this.room.hostIsMe) {
                    await this._handleCheckColor(rpc);
                }
                break;
            case constant_1.RpcMessageTag.SetName:
                await this._handleSetName(rpc);
                break;
            case constant_1.RpcMessageTag.SetColor:
                await this._handleSetColor(rpc);
                break;
            case constant_1.RpcMessageTag.ReportDeadBody:
                if (this.room.hostIsMe) {
                    await this._handleReportDeadBody(rpc);
                }
                break;
            case constant_1.RpcMessageTag.MurderPlayer:
                await this._handleMurderPlayer(rpc);
                break;
            case constant_1.RpcMessageTag.SendChat:
                await this._handleSendChat(rpc);
                break;
            case constant_1.RpcMessageTag.StartMeeting:
                await this._handleStartMeeting(rpc);
                break;
            case constant_1.RpcMessageTag.SetStartCounter:
                await this._handleSetStartCounter(rpc);
                break;
            case constant_1.RpcMessageTag.UsePlatform:
                await this._handleUsePlatform(rpc);
                break;
            case constant_1.RpcMessageTag.SendQuickChat:
                await this._handleSendQuickChat(rpc);
                break;
            case constant_1.RpcMessageTag.SetHat:
                await this._handleSetHat(rpc);
                break;
            case constant_1.RpcMessageTag.SetSkin:
                await this._handleSetSkin(rpc);
                break;
            case constant_1.RpcMessageTag.SetPet:
                await this._handleSetPet(rpc);
                break;
            case constant_1.RpcMessageTag.SetVisor:
                await this._handleSetVisor(rpc);
                break;
            case constant_1.RpcMessageTag.SetNameplate:
                await this._handleSetNameplate(rpc);
                break;
            case constant_1.RpcMessageTag.SetRole:
                await this._handleSetRole(rpc);
                break;
            case constant_1.RpcMessageTag.ProtectPlayer:
                await this._handleProtectPlayer(rpc);
                break;
            case constant_1.RpcMessageTag.Shapeshift:
                await this._handleShapeshift(rpc);
                break;
            case constant_1.RpcMessageTag.CheckMurder:
                await this._handleCheckMurder(rpc);
                break;
            case constant_1.RpcMessageTag.CheckProtect:
                await this._handleCheckProtect(rpc);
                break;
        }
    }
    async _handleCompleteTask(rpc) {
        var _a;
        if (!((_a = this.player.playerInfo) === null || _a === void 0 ? void 0 : _a.taskStates))
            return;
        this._completeTask(rpc.taskIdx);
        await this.emit(new events_1.PlayerCompleteTaskEvent(this.room, this.player, rpc, this.player.playerInfo.taskStates[rpc.taskIdx]));
    }
    _completeTask(taskIdx) {
        var _a;
        (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.completeTask(this.playerId, taskIdx);
    }
    async _rpcCompleteTask(taskIdx) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.CompleteTaskMessage(taskIdx)));
    }
    /**
     * Mark one of this player's tasks as completed.
     * @param taskIdx The index of the task to complete. Note: this is not the
     * task ID but instead the index of the player's task, as in its position in
     * the {@link PlayerInfo.taskStates} array.
     *
     * Emits a {@link PlayerCompleteTaskEvent | `player.completetask`} event.
     *
     * @example
     * ```ts
     * client.me.control.completeTask(0);
     * ```
     */
    completeTask(taskIdx) {
        var _a;
        if (!((_a = this.player.playerInfo) === null || _a === void 0 ? void 0 : _a.taskStates[taskIdx]))
            return;
        this.emitSync(new events_1.PlayerCompleteTaskEvent(this.room, this.player, undefined, this.player.playerInfo.taskStates[taskIdx]));
        this._completeTask(taskIdx);
        this._rpcCompleteTask(taskIdx);
    }
    async _handleSyncSettings(rpc) {
        if (!rpc.settings)
            return;
        this._syncSettings(rpc.settings);
        const ev = await this.emit(new events_1.PlayerSyncSettingsEvent(this.room, this.player, rpc, rpc.settings));
        if (ev.isDirty) {
            this._syncSettings(ev.alteredSettings);
            this._rpcSyncSettings(ev.alteredSettings);
        }
    }
    _syncSettings(settings) {
        this.room.settings.patch(settings);
    }
    async _rpcSyncSettings(settings) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SyncSettingsMessage(settings)));
    }
    /**
     * Change the room's settings. Use {@link Hostable.setSettings} to pass a
     * partial game objects object. This is a host operation on official servers.
     *
     * Emits a {@link PlayerSyncSettingsEvent | `player.syncsettings`} event.
     *
     * @property settings The full game settings object to update the settings to.
     */
    async syncSettings(settings) {
        this._syncSettings(settings);
        const ev = await this.emit(new events_1.PlayerSyncSettingsEvent(this.room, this.player, undefined, settings));
        this._syncSettings(ev.alteredSettings);
        this._rpcSyncSettings(settings);
    }
    async _handleCheckName(rpc) {
        if (!this.room.gameData)
            return;
        let newName = rpc.name;
        const players = [...this.room.gameData.players.values()];
        if (players.some((player) => player.playerId !== this.playerId &&
            player.defaultOutfit.name.toLowerCase() === newName.toLowerCase())) {
            for (let i = 1; i < 100; i++) {
                newName = rpc.name + " " + i;
                if (!players.some((player) => player.playerId !== this.playerId &&
                    player.defaultOutfit.name.toLowerCase() === newName.toLowerCase())) {
                    break;
                }
            }
        }
        const ev = await this.emit(new events_1.PlayerCheckNameEvent(this.room, this.player, rpc, rpc.name, newName));
        if (!ev.canceled)
            await this.setName(ev.alteredName);
    }
    async _rpcCheckName(name) {
        await this.room.broadcast([
            new protocol_1.RpcMessage(this.netId, new protocol_1.CheckNameMessage(name)),
        ], undefined, [this.room.hostId]);
    }
    /**
     * Request for the host to check a name for this player and append numbers if
     * it's taken.
     * @param name The name to request.
     */
    async checkName(name) {
        await this._rpcCheckName(name);
    }
    async _handleSetName(rpc) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldName = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.name;
        if (defaultOutfit)
            defaultOutfit.name = rpc.name;
        const ev = await this.emit(new events_1.PlayerSetNameEvent(this.room, this.player, rpc, oldName || "", rpc.name));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setName(constant_1.PlayerOutfitType.Default, ev.alteredName);
        if (playerInfo && this.room.hostIsMe) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredName !== rpc.name)
            this._rpcSetName(ev.alteredName);
    }
    _rpcSetName(name) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetNameMessage(name)));
    }
    /**
     * Update this player's name. This is a host operation on official servers.
     * Use {@link PlayerControl.checkName} if you are calling this as not the host.
     *
     * Emits a {@link PlayerSetNameEvent | `player.setname`} event.
     *
     * @param name The name to set this player's name to.
     */
    async setName(name) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldName = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.name;
        if (defaultOutfit)
            defaultOutfit.name = name;
        const ev = await this.emit(new events_1.PlayerSetNameEvent(this.room, this.player, undefined, oldName || "", name));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setName(constant_1.PlayerOutfitType.Default, ev.alteredName);
        if (playerInfo) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredName !== oldName)
            this._rpcSetName(ev.alteredName);
    }
    async _handleCheckColor(rpc) {
        if (!this.room.gameData)
            return;
        let newColor = rpc.color;
        const players = [...this.room.gameData.players.values()];
        let i = 0;
        while (players.some((player) => player.playerId !== this.playerId &&
            player.defaultOutfit.color === newColor)) {
            newColor++;
            if (newColor >= 18)
                newColor = 0;
            i++;
            if (i >= 18)
                break;
        }
        const ev = await this.emit(new events_1.PlayerCheckColorEvent(this.room, this.player, rpc, rpc.color, newColor));
        if (!ev.canceled)
            this.setColor(ev.alteredColor);
    }
    async _rpcCheckColor(color) {
        await this.room.broadcast([
            new protocol_1.RpcMessage(this.netId, new protocol_1.CheckColorMessage(color)),
        ], undefined, [this.room.hostId]);
    }
    /**
     * Request for the host to check a color for this player and change it if it's
     * taken.
     *
     * @param color The color to request.
     */
    async checkColor(color) {
        await this._rpcCheckColor(color);
    }
    async _handleSetColor(rpc) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldColor = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.color;
        if (defaultOutfit)
            defaultOutfit.color = rpc.color;
        const ev = await this.emit(new events_1.PlayerSetColorEvent(this.room, this.player, rpc, oldColor || constant_1.Color.Red, rpc.color));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setColor(constant_1.PlayerOutfitType.Default, ev.alteredColor);
        if (playerInfo && this.room.hostIsMe) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredColor !== rpc.color)
            this._rpcSetColor(ev.alteredColor);
    }
    _rpcSetColor(color) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetColorMessage(color)));
    }
    /**
     * Update this player's color. This is a host operation on official servers.
     * Use {@link PlayerControl.checkColor} if you are calling this as not the host.
     *
     * Emits a {@link PlayerCheckNameEvent | `player.checkname`} event.
     *
     * @param color The color to set this player's name to.
     */
    async setColor(color) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldColor = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.color;
        if (defaultOutfit)
            defaultOutfit.color = color;
        const ev = await this.emit(new events_1.PlayerSetColorEvent(this.room, this.player, undefined, oldColor || constant_1.Color.Red, color));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setColor(constant_1.PlayerOutfitType.Default, ev.alteredColor);
        if (playerInfo) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredColor !== oldColor) {
            this._rpcSetColor(ev.alteredColor);
        }
    }
    async _handleReportDeadBody(rpc) {
        var _a, _b;
        const reportedBody = rpc.bodyid === 0xff
            ? "emergency"
            : this.room.getPlayerByPlayerId(rpc.bodyid);
        if (!reportedBody)
            return;
        const ev = await this.emit(new events_1.PlayerReportDeadBodyEvent(this.room, this.player, rpc, reportedBody));
        if (!ev.canceled) {
            (_b = (_a = this.room.host) === null || _a === void 0 ? void 0 : _a.control) === null || _b === void 0 ? void 0 : _b.startMeeting(ev.alteredBody, this.player);
        }
    }
    async _rpcReportDeadBody(body) {
        if (body !== "emergency" && body.playerId === undefined) {
            return this._rpcReportDeadBody("emergency");
        }
        await this.room.broadcast([
            new protocol_1.RpcMessage(this.netId, new protocol_1.ReportDeadBodyMessage(body === "emergency"
                ? 0xff
                : body.playerId))
        ], undefined, [this.room.hostId]);
    }
    async kill(reason) {
        var _a;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setDead(true);
        const ev = await this.emit(new events_1.PlayerDieEvent(this.room, this.player, reason));
        if (ev.reverted) {
            playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setDead(false);
        }
    }
    async _handleMurderPlayer(rpc) {
        var _a, _b, _c;
        const victim = this.room.getPlayerByNetId(rpc.victimNetId);
        if (!victim)
            return;
        await ((_a = victim.control) === null || _a === void 0 ? void 0 : _a.kill("murder"));
        await this.emit(new events_1.PlayerMurderEvent(this.room, this.player, rpc, victim));
        this._checkMurderEndGame();
        if (((_b = victim.playerInfo) === null || _b === void 0 ? void 0 : _b.isDead) && this.room.hostIsMe) {
            await ((_c = this.room.shipStatus) === null || _c === void 0 ? void 0 : _c.tryAssignGhostRole(victim));
        }
    }
    _rpcMurderPlayer(victim) {
        if (!victim.control)
            return;
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.MurderPlayerMessage(victim.control.netId)));
    }
    /**
     * Murder another a player. This operation can only be called if the player
     * is the impostor on official servers.
     *
     * Due to technical limitations, this operation cannot be canceled or reverted
     * without advanced "breaking game", therefore it is out of scope of a single
     * `.revert()` function on the event emitted.
     *
     * Emits a {@link PlayerMurderEvent | `player.murder`} event.
     *
     * @param victim The player to murder.
     * @returns
     */
    async murderPlayer(victim) {
        var _a, _b, _c, _d;
        if (!this.room.hostIsMe) {
            await this._rpcCheckMurder(victim);
            return;
        }
        if ((_a = victim.control) === null || _a === void 0 ? void 0 : _a.protectedByGuardian) {
            this._rpcMurderPlayer(victim);
            await victim.control.removeProtection();
            return;
        }
        await ((_b = victim.control) === null || _b === void 0 ? void 0 : _b.kill("murder"));
        await this.emit(new events_1.PlayerMurderEvent(this.room, this.player, undefined, victim));
        this._rpcMurderPlayer(victim);
        this._checkMurderEndGame();
        if (((_c = victim.playerInfo) === null || _c === void 0 ? void 0 : _c.isDead) && this.room.hostIsMe) {
            await ((_d = this.room.shipStatus) === null || _d === void 0 ? void 0 : _d.tryAssignGhostRole(victim));
        }
    }
    _checkMurderEndGame(victim) {
        if (!this.room.gameData)
            return;
        let aliveCrewmates = 0;
        let aliveImpostors = 0;
        for (const [, playerInfo] of this.room.gameData.players) {
            if (!playerInfo.isDisconnected && !playerInfo.isDead) {
                if (playerInfo.isImpostor) {
                    aliveImpostors++;
                }
                else {
                    aliveCrewmates++;
                }
            }
        }
        if (aliveCrewmates <= aliveImpostors) {
            this.room.registerEndGameIntent(new endgame_1.EndGameIntent(endgame_1.AmongUsEndGames.PlayersKill, constant_1.GameOverReason.ImpostorByKill, {
                killer: this.player,
                victim,
                aliveCrewmates,
                aliveImpostors
            }));
        }
    }
    async _handleSendChat(rpc) {
        await this.emit(new events_1.PlayerSendChatEvent(this.room, this.player, rpc, rpc.message));
    }
    _rpcSendChat(message) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SendChatMessage(message)));
    }
    /**
     * Send a chat message as this player.
     *
     * Due to technical impossibilities, this event cannot be canceled or reverted.
     *
     * Emits a {@link PlayerSendChatEvent | `player.chat`} event.
     */
    sendChat(message) {
        this.emitSync(new events_1.PlayerSendChatEvent(this.room, this.player, undefined, message));
        this._rpcSendChat(message);
    }
    _rpcSendChatNote(player, type) {
        if (player.playerId === undefined)
            return;
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SendChatNoteMessage(player.playerId, type)));
    }
    /**
     * Send a player chat note for this player, i.e. "player x voted, x remaining.".
     */
    sendChatNote(player, type) {
        this._rpcSendChatNote(player, type);
    }
    async _handleStartMeeting(rpc) {
        const reportedBody = rpc.bodyid === 0xff
            ? "emergency"
            : this.room.getPlayerByPlayerId(rpc.bodyid) || "emergency";
        await this.emit(new events_1.PlayerStartMeetingEvent(this.room, this.player, rpc, this.player, reportedBody));
        if (this.room.hostIsMe) {
            this._startMeeting(this.player);
        }
    }
    _startMeeting(caller) {
        var _a;
        if (caller.playerId === undefined)
            return;
        const spawnMeetinghud = this.room.spawnPrefabOfType(constant_1.SpawnType.MeetingHud, -2, undefined, undefined, false);
        const callerState = spawnMeetinghud.voteStates.get(caller.playerId);
        if (callerState) {
            callerState.didReport = true;
        }
        this.room.messageStream.push(new protocol_1.SpawnMessage(spawnMeetinghud.spawnType, -2, 0, spawnMeetinghud.components.map(component => {
            const writer = util_1.HazelWriter.alloc(512);
            writer.write(component, true);
            writer.realloc(writer.cursor);
            return new protocol_1.ComponentSpawnData(component.netId, writer.buffer);
        })));
        const movingPlatform = (_a = this.room.shipStatus) === null || _a === void 0 ? void 0 : _a.systems.get(constant_1.SystemType.GapRoom);
        if (movingPlatform instanceof systems_1.MovingPlatformSystem) {
            movingPlatform.setSide(systems_1.MovingPlatformSide.Left);
        }
    }
    _rpcStartMeeting(player) {
        if (player !== "emergency" && player.playerId === undefined) {
            return this._rpcStartMeeting("emergency");
        }
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.StartMeetingMessage(player === "emergency"
            ? 0xff
            : player.playerId)));
    }
    /**
     * If you're the host, this will immediately begin a meeting
     * Start a meeting and begin the meeting
     *
     * Emits a {@link PlayerStartMeetingEvent | `player.startmeeting`} event.
     *
     * @param body The body that was reported, or "emergency" if it is an emergency meeting.
     * @param caller The player that called this meeting.
     */
    async startMeeting(body, caller) {
        if (!this.room.hostIsMe) {
            await this._rpcReportDeadBody(body);
            return;
        }
        const actualCaller = caller || this.room.host || this.player;
        this._rpcStartMeeting(body);
        this._startMeeting(actualCaller);
        await this.emit(new events_1.PlayerStartMeetingEvent(this.room, this.player, undefined, actualCaller, body));
    }
    async _handleSetStartCounter(rpc) {
        const oldCounter = this.room.counter;
        this._setStartCounter(rpc.counter);
        const ev = await this.emit(new events_1.PlayerSetStartCounterEvent(this.room, this.player, rpc, oldCounter, rpc.counter));
        this.room.counter = ev.alteredCounter;
        if (this.room.counter !== rpc.counter) {
            await this._rpcSetStartCounter(ev.alteredCounter);
        }
    }
    _setStartCounter(counter) {
        this.lastStartCounter++;
        this.room.counter = counter;
    }
    _rpcSetStartCounter(counter) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetStartCounterMessage(this.lastStartCounter, counter)));
    }
    /**
     * Change the counter at the bottom of the screen while in the lobby,
     * usually counting down from 5 to 1. This is a host-only operation
     * on official servers.
     *
     * Emits a {@link PlayerSetStartCounterEvent | `player.setstartcounter`} event.
     *
     * @param counter The counter to set to.
     */
    async setStartCounter(counter) {
        this._setStartCounter(counter);
        const ev = await this.emit(new events_1.PlayerSetStartCounterEvent(this.room, this.player, undefined, this.room.counter, counter));
        if (ev.alteredCounter !== counter) {
            this.room.counter = ev.alteredCounter;
        }
        this._rpcSetStartCounter(ev.alteredCounter);
    }
    async _handleUsePlatform(rpc) {
        const airship = this.room.shipStatus;
        if (!airship || !this.room.hostIsMe)
            return;
        const ev = await this.emit(new events_1.PlayerUseMovingPlatformEvent(this.room, this.player, rpc));
        if (ev.canceled)
            return;
        this._usePlatform(false);
    }
    _usePlatform(rpc) {
        const airship = this.room.shipStatus;
        if (!airship)
            return;
        const movingPlatform = airship.systems.get(constant_1.SystemType.GapRoom);
        if (movingPlatform instanceof systems_1.MovingPlatformSystem) {
            movingPlatform.setTarget(this.player, movingPlatform.oppositeSide, rpc);
        }
    }
    /**
     * Use the moving platfrom on the map, i.e. the one on Airship.
     *
     * Emits a {@link MovingPlatformPlayerUpdateEvent | `movingplatform.playerupdate`} event.
     */
    async usePlatform() {
        const ev = await this.emit(new events_1.PlayerUseMovingPlatformEvent(this.room, this.player, undefined));
        if (ev.canceled)
            return;
        this._usePlatform(true);
    }
    async _handleSendQuickChat(rpc) {
        await this.emit(new events_1.PlayerSendQuickChatEvent(this.room, this.player, rpc, rpc.message));
    }
    _rpcSendQuickChat(message) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SendQuickChatMessage(message)));
    }
    /**
     * Send a quick chat message as this player.
     *
     * Due to technical impossibilities, this event cannot be canceled or reverted.
     *
     * Emits a {@link PlayerSendChatEvent | `player.quickchat`} event.
     */
    sendQuickChat(message, format) {
        const quickChatMessage = typeof message === "number"
            ? format
                ? new protocol_1.QuickChatSentenceMessageData(message, format.map(format => {
                    return typeof format === "number"
                        ? format
                        : new protocol_1.QuickChatPlayerMessageData(format.playerId);
                }))
                : new protocol_1.QuickChatPhraseMessageData(message)
            : new protocol_1.QuickChatPlayerMessageData(message.playerId);
        this.emitSync(new events_1.PlayerSendQuickChatEvent(this.room, this.player, undefined, quickChatMessage));
        this._rpcSendQuickChat(quickChatMessage);
    }
    async _handleSetHat(rpc) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldHat = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.hatId;
        if (defaultOutfit)
            defaultOutfit.hatId = rpc.hatId;
        const ev = await this.emit(new events_1.PlayerSetHatEvent(this.room, this.player, rpc, oldHat || constant_1.Hat.NoHat, rpc.hatId));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setHat(constant_1.PlayerOutfitType.Default, ev.alteredHatId);
        if (playerInfo && this.room.hostIsMe) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredHatId !== rpc.hatId)
            this._rpcSetHat(ev.alteredHatId);
    }
    _rpcSetHat(hatId) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetHatMessage(hatId)));
    }
    /**
     * Update this player's hat. This is not a host operation unless the
     * client does not own this player.
     *
     * Emits a {@link PlayerSetHatEvent | `player.sethat`} event.
     *
     * @param hatId The hat to set this player's hat to, see {@link Hat}.
     */
    async setHat(hatId) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldHat = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.hatId;
        if (defaultOutfit)
            defaultOutfit.hatId = hatId;
        const ev = await this.emit(new events_1.PlayerSetHatEvent(this.room, this.player, undefined, oldHat || constant_1.Hat.NoHat, hatId));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setHat(constant_1.PlayerOutfitType.Default, ev.alteredHatId);
        if (playerInfo) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredHatId !== oldHat)
            this._rpcSetHat(ev.alteredHatId);
    }
    async _handleSetSkin(rpc) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldSkin = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.skinId;
        if (defaultOutfit)
            defaultOutfit.skinId = rpc.skinId;
        const ev = await this.emit(new events_1.PlayerSetSkinEvent(this.room, this.player, rpc, oldSkin || constant_1.Skin.None, rpc.skinId));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setSkin(constant_1.PlayerOutfitType.Default, ev.alteredSkin);
        if (playerInfo && this.room.hostIsMe) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredSkin !== rpc.skinId)
            this._rpcSetSkin(ev.alteredSkin);
    }
    _rpcSetSkin(skinId) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetSkinMessage(skinId)));
    }
    /**
     * Update this player's skin. This is not a host operation unless the
     * client does not own this player.
     *
     * Emits a {@link PlayerSetSkinEvent | `player.setskin`} event.
     *
     * @param skinId The skin to set this player's skin to, see {@link Skin}.
     */
    async setSkin(skinId) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldSkin = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.skinId;
        if (defaultOutfit)
            defaultOutfit.skinId = skinId;
        const ev = await this.emit(new events_1.PlayerSetSkinEvent(this.room, this.player, undefined, oldSkin || constant_1.Skin.None, skinId));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setSkin(constant_1.PlayerOutfitType.Default, ev.alteredSkin);
        if (playerInfo) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredSkin !== oldSkin)
            this._rpcSetSkin(ev.alteredSkin);
    }
    async _handleSetPet(rpc) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldPet = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.petId;
        if (defaultOutfit)
            defaultOutfit.petId = rpc.petId;
        const ev = await this.emit(new events_1.PlayerSetPetEvent(this.room, this.player, rpc, oldPet || constant_1.Pet.EmptyPet, rpc.petId));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setPet(constant_1.PlayerOutfitType.Default, ev.alteredPetId);
        if (playerInfo && this.room.hostIsMe) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredPetId !== rpc.petId)
            this._rpcSetPet(ev.alteredPetId);
    }
    _rpcSetPet(petId) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetPetMessage(petId)));
    }
    /**
     * Update this player's pet. This is not a host operation unless the
     * client does not own this player.
     *
     * Emits a {@link PlayerSetPetEvent | `player.setpet`} event.
     *
     * @param petId The pet to set this player's pet to, see {@link Pet}.
     */
    async setPet(petId) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldPet = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.petId;
        if (defaultOutfit)
            defaultOutfit.petId = petId;
        const ev = await this.emit(new events_1.PlayerSetPetEvent(this.room, this.player, undefined, oldPet || constant_1.Pet.EmptyPet, petId));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setPet(constant_1.PlayerOutfitType.Default, ev.alteredPetId);
        if (playerInfo) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredPetId !== oldPet)
            this._rpcSetPet(ev.alteredPetId);
    }
    async _handleSetVisor(rpc) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldVisor = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.visorId;
        if (defaultOutfit)
            defaultOutfit.visorId = rpc.visorId;
        const ev = await this.emit(new events_1.PlayerSetVisorEvent(this.room, this.player, rpc, oldVisor || constant_1.Visor.EmptyVisor, rpc.visorId));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setVisor(constant_1.PlayerOutfitType.Default, ev.alteredVisorId);
        if (playerInfo && this.room.hostIsMe) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredVisorId !== rpc.visorId)
            this._rpcSetVisor(ev.alteredVisorId);
    }
    _rpcSetVisor(visorId) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetVisorMessage(visorId)));
    }
    /**
     * Update this player's visor. This is not a host operation unless the
     * client does not own this player.
     *
     * Emits a {@link PlayerSetVisorEvent | `player.setvisor`} event.
     *
     * @param visorId The visor to set this player's visor to, see {@link Visor}.
     */
    async setVisor(visorId) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldVisor = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.visorId;
        if (defaultOutfit)
            defaultOutfit.visorId = visorId;
        const ev = await this.emit(new events_1.PlayerSetVisorEvent(this.room, this.player, undefined, oldVisor || constant_1.Visor.EmptyVisor, visorId));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setVisor(constant_1.PlayerOutfitType.Default, ev.alteredVisorId);
        if (playerInfo) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredVisorId !== oldVisor)
            this._rpcSetVisor(ev.alteredVisorId);
    }
    async _handleSetNameplate(rpc) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldNameplate = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.nameplateId;
        if (defaultOutfit)
            defaultOutfit.nameplateId = rpc.nameplateId;
        const ev = await this.emit(new events_1.PlayerSetNameplateEvent(this.room, this.player, rpc, oldNameplate || constant_1.Nameplate.NoPlate, rpc.nameplateId));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setNameplate(constant_1.PlayerOutfitType.Default, ev.alteredNameplateId);
        if (playerInfo && this.room.hostIsMe) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredNameplateId !== rpc.nameplateId)
            this._rpcSetNameplate(ev.alteredNameplateId);
    }
    _rpcSetNameplate(nameplateId) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetNameplateMessage(nameplateId)));
    }
    /**
     * Update this player's nameplate. This is not a host operation unless the
     * client does not own this player.
     *
     * Emits a {@link PlayerSetNameplateEvent | `player.setnameplate`} event.
     *
     * @param nameplateId The nameplate to set this player's nameplate to, see {@link Nameplate}.
     */
    async setNameplate(nameplateId) {
        var _a, _b;
        const playerInfo = (_a = this.room.gameData) === null || _a === void 0 ? void 0 : _a.players.get(this.playerId);
        const defaultOutfit = playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.defaultOutfit;
        const oldNameplate = defaultOutfit === null || defaultOutfit === void 0 ? void 0 : defaultOutfit.nameplateId;
        if (defaultOutfit)
            defaultOutfit.nameplateId = nameplateId;
        const ev = await this.emit(new events_1.PlayerSetNameplateEvent(this.room, this.player, undefined, oldNameplate || constant_1.Nameplate.NoPlate, nameplateId));
        playerInfo === null || playerInfo === void 0 ? void 0 : playerInfo.setNameplate(constant_1.PlayerOutfitType.Default, ev.alteredNameplateId);
        if (playerInfo) {
            (_b = this.room.gameData) === null || _b === void 0 ? void 0 : _b.markDirty(this.playerId);
        }
        if (ev.alteredNameplateId !== oldNameplate)
            this._rpcSetNameplate(ev.alteredNameplateId);
    }
    async _handleSetRole(rpc) {
        var _a;
        const playerInfo = this.player.playerInfo;
        if (!playerInfo)
            return;
        const oldRoleType = playerInfo.roleType;
        const newRole = this.room.registeredRoles.get(rpc.roleType) || roles_1.UnknownRole(rpc.roleType);
        playerInfo.roleType = newRole;
        this.player.role = new newRole(this.player);
        const ev = await this.emit(new events_1.PlayerSetRoleEvent(this.room, this.player, rpc, oldRoleType, newRole));
        if (ev.alteredRole !== newRole) {
            this._rpcSetRole(((_a = ev.alteredRole) === null || _a === void 0 ? void 0 : _a.roleMetadata.roleType) || constant_1.RoleType.Crewmate);
            playerInfo.roleType = ev.alteredRole;
            this.player.role = new ev.alteredRole(this.player);
        }
        await this.player.role.onInitialize();
    }
    _rpcSetRole(roleType) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SetRoleMessage(roleType)));
    }
    async setRole(roleCtr) {
        var _a;
        const playerInfo = this.player.playerInfo;
        if (!playerInfo)
            return;
        const oldRoleType = playerInfo.roleType;
        playerInfo.roleType = roleCtr;
        this.player.role = new roleCtr(this.player);
        const ev = await this.emit(new events_1.PlayerSetRoleEvent(this.room, this.player, undefined, oldRoleType, roleCtr));
        if (ev.alteredRole !== roleCtr) {
            playerInfo.roleType = ev.alteredRole;
            this.player.role = new ev.alteredRole(this.player);
        }
        this._rpcSetRole(((_a = ev.alteredRole) === null || _a === void 0 ? void 0 : _a.roleMetadata.roleType) || constant_1.RoleType.Crewmate);
        await this.player.role.onInitialize();
    }
    async _handleProtectPlayer(rpc) {
        var _a;
        const target = this.room.getPlayerByNetId(rpc.targetNetId);
        if (!target)
            return;
        (_a = target.control) === null || _a === void 0 ? void 0 : _a._addProtection(this.player);
        await this.emit(new events_1.PlayerProtectEvent(this.room, this.player, rpc, target, this.room.settings.roleSettings.protectionDurationSeconds));
    }
    _addProtection(guardianProtector) {
        this.protectedByGuardian = true;
        this.guardianProtector = guardianProtector;
        this._protectedByGuardianTime = this.room.settings.roleSettings.protectionDurationSeconds;
    }
    _rpcProtectPlayer(target, angelColor) {
        if (!target.control)
            return;
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.ProtectPlayerMessage(target.control.netId, angelColor)));
    }
    async protectPlayer(target, angelColor) {
        var _a, _b;
        if (angelColor === void 0) { angelColor = ((_a = this.player.playerInfo) === null || _a === void 0 ? void 0 : _a.defaultOutfit.color) || constant_1.Color.Red; }
        if (!this.room.hostIsMe) {
            await this._rpcCheckProtect(target);
            return;
        }
        (_b = target.control) === null || _b === void 0 ? void 0 : _b._addProtection(this.player);
        this._rpcProtectPlayer(target, angelColor);
        await this.emit(new events_1.PlayerProtectEvent(this.room, this.player, undefined, target, this.room.settings.roleSettings.protectionDurationSeconds));
    }
    async _handleShapeshift(rpc) {
        const target = this.room.getPlayerByNetId(rpc.targetNetId);
        if (!target)
            return;
        const oldTarget = this.shapeshiftTarget;
        if (target === this.player) {
            if (!oldTarget)
                return;
            this._shapeshift(target);
            const ev = await this.emit(new events_1.PlayerRevertShapeshiftEvent(this.room, this.player, undefined, oldTarget, rpc.doAnimation));
            if (ev.reverted) {
                this._shapeshift(oldTarget);
                this._rpcShapeshift(oldTarget, ev.alteredDoAnimation);
                return;
            }
        }
        else {
            this._shapeshift(target);
            const ev = await this.emit(new events_1.PlayerShapeshiftEvent(this.room, this.player, undefined, target, this.room.settings.roleSettings.shapeshiftDuration, rpc.doAnimation));
            if (ev.reverted) {
                this._shapeshift(this.player);
                this._rpcShapeshift(target, ev.alteredDoAnimation);
                return;
            }
            if (ev.alteredTarget !== target) {
                this._shapeshift(ev.alteredTarget);
                this._rpcShapeshift(ev.alteredTarget, ev.alteredDoAnimation);
                return;
            }
        }
    }
    _shapeshift(target) {
        this.shapeshiftTarget = target === this.player
            ? undefined
            : target;
        if (!target.playerInfo || !this.player.playerInfo)
            return;
        if (target === this.player) {
            this.player.playerInfo.deleteOutfit(constant_1.PlayerOutfitType.Shapeshifter);
        }
        else {
            this.player.playerInfo.setOutfit(target.playerInfo.defaultOutfit.clone(constant_1.PlayerOutfitType.Shapeshifter));
            this.player.playerInfo.currentOutfitType = constant_1.PlayerOutfitType.Shapeshifter;
        }
    }
    _rpcShapeshift(target, doAnimation) {
        if (!target.control)
            return;
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.ShapeshiftMessage(target.control.netId, doAnimation)));
    }
    async shapeshift(target, doAnimation = true) {
        const oldTarget = this.shapeshiftTarget;
        if (target === this.player) {
            if (!oldTarget)
                return;
            this._shapeshift(target);
            const ev = await this.emit(new events_1.PlayerRevertShapeshiftEvent(this.room, this.player, undefined, oldTarget, doAnimation));
            if (ev.reverted) {
                this._shapeshift(oldTarget);
                return;
            }
            this._rpcShapeshift(target, ev.alteredDoAnimation);
        }
        else {
            this._shapeshift(target);
            const ev = await this.emit(new events_1.PlayerShapeshiftEvent(this.room, this.player, undefined, target, this.room.settings.roleSettings.shapeshiftDuration, doAnimation));
            if (ev.reverted) {
                this._shapeshift(this.player);
                return;
            }
            if (ev.alteredTarget !== target) {
                this._shapeshift(ev.alteredTarget);
                this._rpcShapeshift(ev.alteredTarget, ev.alteredDoAnimation);
                return;
            }
            this._rpcShapeshift(target, doAnimation);
        }
    }
    async revertShapeshift() {
        await this.shapeshift(this.player);
    }
    async _handleCheckMurder(rpc) {
        var _a;
        const victim = this.room.getPlayerByNetId(rpc.victimNetId);
        if (!victim || !this.room.hostIsMe)
            return;
        const ev = await this.emit(new events_1.PlayerCheckMurderEvent(this.room, this.player, rpc, victim, this.room.murderIsValid(this.player, victim)));
        if (ev.alteredIsValid) {
            (_a = ev.alteredPlayer.control) === null || _a === void 0 ? void 0 : _a.murderPlayer(ev.alteredVictim);
        }
    }
    async _rpcCheckMurder(victim) {
        if (!victim.control)
            return;
        await this.room.broadcast([
            new protocol_1.RpcMessage(this.netId, new protocol_1.CheckMurderMessage(victim.control.netId))
        ]);
    }
    /**
     * Remove this player's protection from a guardian angel, if they have one.
     *
     * Emits a {@link PlayerRemoveProtectionEvent | `player.removeprotection`} event.
     * This is not networked, so other players will not see this update.
     *
     * @param timeout Whether or not this is because of a timeout.
     */
    async removeProtection(timeout = false) {
        if (!this.protectedByGuardian)
            return;
        const oldGuardian = this.guardianProtector;
        this.protectedByGuardian = false;
        this.guardianProtector = undefined;
        const ev = await this.emit(new events_1.PlayerRemoveProtectionEvent(this.room, this.player, true));
        if (ev.reverted) {
            this.protectedByGuardian = true;
            this.guardianProtector = oldGuardian;
            // In case this event was emitted when the timer ran out & it was reverted,
            // we can ensure that it would never be called again for timer running out
            // by setting the timer to Infinity.
            this._protectedByGuardianTime = Infinity;
        }
    }
    _protectIsValid(target) {
        var _a, _b;
        if (this.room.gameState !== constant_1.GameState.Started)
            return false;
        if (((_a = this.player.playerInfo) === null || _a === void 0 ? void 0 : _a.roleType) !== roles_1.GuardianAngelRole || this.player.playerInfo.isImpostor || this.player.playerInfo.isDisconnected)
            return false;
        if (!target.playerInfo || ((_b = target.playerInfo) === null || _b === void 0 ? void 0 : _b.isDead))
            return false;
        return true;
    }
    async _handleCheckProtect(rpc) {
        var _a;
        const target = this.room.getPlayerByNetId(rpc.targetNetId);
        if (!target || !this.room.hostIsMe)
            return;
        const ev = await this.emit(new events_1.PlayerCheckProtectEvent(this.room, this.player, rpc, target, this._protectIsValid(target)));
        if (ev.alteredIsValid) {
            await ((_a = ev.alteredPlayer.control) === null || _a === void 0 ? void 0 : _a.protectPlayer(ev.alteredTarget));
        }
    }
    async _rpcCheckProtect(target) {
        var _a;
        if (!target.control)
            return;
        await this.room.broadcast([
            new protocol_1.RpcMessage(this.netId, new protocol_1.ProtectPlayerMessage(target.control.netId, ((_a = target.playerInfo) === null || _a === void 0 ? void 0 : _a.defaultOutfit.color) || constant_1.Color.Red))
        ]);
    }
}
exports.PlayerControl = PlayerControl;
//# sourceMappingURL=PlayerControl.js.map